package data_input

import (
	"encoding/json"
	"fmt"
	"hongze/equity_admin/controllers"
	"hongze/equity_admin/models"
	"hongze/equity_admin/models/data_input"
	"hongze/equity_admin/services"
	"hongze/equity_admin/utils"
	"rdluck_tools/paging"
	"sort"
	"strings"
	"time"
)

type DataInputController struct {
	controllers.BaseAuthController
}

// @Title 获取所有行业分类
// @Description 获取所有行业分类接口
// @Success 200 {object} data_input.AllIndustryResp
// @router /industry/all/list [get]
func (this *DataInputController) IndustryAllList() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()

	sysUser := this.SysUser
	if sysUser == nil {
		br.Msg = "请登录"
		br.ErrMsg = "请登录，SysUser Is Empty"
		br.Ret = 408
		return
	}

	list, err := data_input.GetAllIndustry()
	if err != nil {
		br.Msg = "获取失败"
		br.ErrMsg = "获取失败，Err:" + err.Error()
		return
	}

	if list == nil {
		list = make([]*data_input.IndustryItem, 0)
	}
	resp := new(data_input.AllIndustryResp)
	resp.List = list
	br.Ret = 200
	br.Success = true
	br.Msg = "获取成功"
	br.Data = resp
}

// @Title 获取用户行业分类列表
// @Description 获取用户行业分类列表接口
// @Success 200 {object} data_input.IndustryResp
// @router /admin/industry/list [get]
func (this *DataInputController) IndustryList() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()

	sysUser := this.SysUser
	if sysUser == nil {
		br.Msg = "请登录"
		br.ErrMsg = "请登录，SysUser Is Empty"
		br.Ret = 408
		return
	}

	var industryStr string
	if sysUser.RoleTypeCode != utils.ROLE_TYPE_CODE_ADMIN {
		adminIndustry, err := data_input.GetAdminIndustryByAdminId(sysUser.AdminId)
		if err != nil && err.Error() != utils.ErrNoRow() {
			br.Msg = "获取失败"
			br.ErrMsg = "获取用户行业信息失败,Err:" + err.Error()
			return
		}
		if adminIndustry == nil {
			br.Msg = "请联系技术开放行业权限"
			br.ErrMsg = "请联系技术开放行业权限,RealName" + sysUser.RealName
			return
		}
		industryArr := strings.Split(adminIndustry.IndustryType, ",")
		for _, v := range industryArr {
			industryStr += `'` + v + `',`
		}
		industryStr = strings.TrimRight(industryStr, ",")
		if industryStr == "" {
			br.Msg = "请联系技术开放行业权限"
			br.ErrMsg = "请联系技术开放行业权限,industryStr is empty,RealName" + sysUser.RealName
			return
		}
	}

	list, err := data_input.GetAdminIndustryList(sysUser.RoleTypeCode, industryStr)
	if err != nil {
		br.Msg = "获取失败"
		br.ErrMsg = "获取失败，Err:" + err.Error()
		return
	}

	if list == nil {
		list = make([]*data_input.Industry, 0)
	}
	resp := new(data_input.IndustryResp)
	resp.List = list
	br.Ret = 200
	br.Success = true
	br.Msg = "获取成功"
	br.Data = resp
}

// @Title 搜索行业分类
// @Description 搜索行业分类接口
// @Param   KeyWord   query   string  false       "搜索关键词"
// @Success 200 {object} data_input.IndustrySearchResp
// @router /industry/search [get]
func (this *DataInputController) IndustrySearch() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()

	sysUser := this.SysUser
	if sysUser == nil {
		br.Msg = "请登录"
		br.ErrMsg = "请登录，SysUser Is Empty"
		br.Ret = 408
		return
	}
	var err error
	keyWord := this.GetString("KeyWord")
	list := make([]*data_input.IndustrySearch, 0)
	if keyWord == "" {
		list, err = data_input.GetAllIndustrySearch()
		if err != nil && err.Error() != utils.ErrNoRow() {
			br.Msg = "获取失败"
			br.ErrMsg = "搜索行业分类信息失败，Err:" + err.Error()
			return
		}
	} else {
		list, err = data_input.GetIndustrySearch(keyWord)
		if err != nil && err.Error() != utils.ErrNoRow() {
			br.Msg = "获取失败"
			br.ErrMsg = "搜索行业分类信息失败，Err:" + err.Error()
			return
		}
	}

	adminIndustry, err := data_input.GetAdminIndustryByAdminId(sysUser.AdminId)
	if err != nil && err.Error() != utils.ErrNoRow() {
		br.Msg = "获取失败"
		br.ErrMsg = "获取用户行业信息失败,Err:" + err.Error()
		return
	}
	if adminIndustry == nil {
		br.Msg = "请联系技术开放行业权限"
		br.ErrMsg = "请联系技术开放行业权限,RealName" + sysUser.RealName
		return
	}

	listLen := len(list)
	for i := 0; i < listLen; i++ {
		item := list[i]
		industryTypeArr := strings.Split(item.IndustryType, "|")
		industryTypeSub := industryTypeArr[0]
		if strings.Contains(adminIndustry.IndustryType, industryTypeSub) {
			list[i].IsAuth = true
		}
	}
	resp := new(data_input.IndustrySearchResp)
	resp.List = list
	br.Ret = 200
	br.Success = true
	br.Msg = "获取成功"
	br.Data = resp
}

// @Title 根据行业获取公司
// @Description 根据行业获取公司接口
// @Param   IndustryType   query   string  false       "行业分类名称"
// @Param   KeyWord   query   string  false       "搜索关键词"
// @Success 200 {object} data_input.CompanyListResp
// @router /company/list [get]
func (this *DataInputController) CompanyList() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()

	sysUser := this.SysUser
	if sysUser == nil {
		br.Msg = "请登录"
		br.ErrMsg = "请登录，SysUser Is Empty"
		br.Ret = 408
		return
	}

	adminIndustry, err := data_input.GetAdminIndustryByAdminId(sysUser.AdminId)
	if err != nil && err.Error() != utils.ErrNoRow() {
		br.Msg = "获取失败"
		br.ErrMsg = "获取用户行业信息失败,Err:" + err.Error()
		return
	}
	if adminIndustry == nil {
		br.Msg = "请联系技术开放行业权限"
		br.ErrMsg = "请联系技术开放行业权限,RealName" + sysUser.RealName
		return
	}

	industryType := this.GetString("IndustryType")
	keyWord := this.GetString("KeyWord")

	var condition string
	var pars []interface{}

	if industryType != "" {
		condition += ` AND a.industry_type=? `
		pars = append(pars, industryType)
	}

	if keyWord != "" {
		condition += ` AND a.trade_name LIKE '%` + keyWord + `%' `
	} else {
		if industryType == "" {
			//获取用户行业
			var industryStr string
			industryArr := strings.Split(adminIndustry.IndustryType, ",")
			for _, v := range industryArr {
				industryStr += `'` + v + `',`
			}
			industryStr = strings.TrimRight(industryStr, ",")
			if industryStr != "" {
				condition += ` AND SUBSTRING_INDEX(a.industry_type,'|',1) IN (` + industryStr + `) `
			}
		}
	}

	list, err := data_input.GetCompanyList(condition, pars)
	if err != nil {
		br.Msg = "获取失败"
		br.ErrMsg = "搜索行业分类信息失败，Err:" + err.Error()
		return
	}

	var companyNameStr string
	for _, v := range list {
		companyNameStr += `'` + v.TradeName + `',`
	}
	companyNameStr = strings.Trim(companyNameStr, ",")

	fmt.Println("companyNameStr:", companyNameStr)

	var fieldCondition string
	var fieldPars []interface{}
	if companyNameStr != "" {
		fieldCondition = ` AND trade_name IN(` + companyNameStr + `)`
	}
	fieldList, err := data_input.GetFieldInfoList(fieldCondition, fieldPars)
	if err != nil {
		fmt.Println("GetFieldInfoList Err:" + err.Error())
	}
	fieldMap := make(map[string][]*data_input.FieldInfo)
	for _, fieldItem := range fieldList {
		needModifyDate, err := services.GetDateByFrequency(fieldItem.DateType)
		if err != nil {
			utils.FileLog.Info("GetDateByFrequency:Err %s", err.Error())
		}
		fieldItem.ModifyDate = needModifyDate
		if _, ok := fieldMap[fieldItem.TradeName]; ok {
			fieldMap[fieldItem.TradeName] = append(fieldMap[fieldItem.TradeName], fieldItem)
		} else {
			fieldMap[fieldItem.TradeName] = append(fieldMap[fieldItem.TradeName], fieldItem)
		}
	}

	lenList := len(list)
	for i := 0; i < lenList; i++ {
		item := list[i]
		if fieldList, ok := fieldMap[item.TradeName]; ok {
			sort.Sort(fieldInfoSlice(fieldList))
			list[i].FieldInfoList = fieldList
		}
		total := 10 //获取公司下待更新指标数据
		list[i].FieldInfoTotal = total

		//判断是否用于此行业权限
		industryTypeArr := strings.Split(item.IndustryType, "|")
		industryTypeSub := industryTypeArr[0]
		if strings.Contains(adminIndustry.IndustryType, industryTypeSub) {
			list[i].IsAuth = true
		}
	}
	sort.Sort(companySlice(list))
	resp := new(data_input.CompanyListResp)
	resp.List = list
	br.Ret = 200
	br.Success = true
	br.Msg = "获取成功"
	br.Data = resp
}

type companySlice []*data_input.CompanyList

func (s companySlice) Len() int { return len(s) }

func (s companySlice) Swap(i, j int) { s[i], s[j] = s[j], s[i] }

func (s companySlice) Less(i, j int) bool { return s[i].FieldInfoTotal >= s[j].FieldInfoTotal }

type fieldInfoSlice []*data_input.FieldInfo

func (s fieldInfoSlice) Len() int { return len(s) }

func (s fieldInfoSlice) Swap(i, j int) { s[i], s[j] = s[j], s[i] }

func (s fieldInfoSlice) Less(i, j int) bool { return s[i].Status <= s[j].Status }

// @Title 获取指标数据
// @Description 获取指标数据接口
// @Param   PageSize   query   int  true       "每页数据条数"
// @Param   CurrentIndex   query   int  true       "当前页页码，从1开始"
// @Param   FieldCode   query   string  false       "指标编码"
// @Param   Sort   query   int  false       "默认0:1：降序，2:升序"
// @Success 200 {object} data_input.FieldDataListResp
// @router /field_data/list [get]
func (this *DataInputController) FieldDataList() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()

	sysUser := this.SysUser
	if sysUser == nil {
		br.Msg = "请登录"
		br.ErrMsg = "请登录，SysUser Is Empty"
		br.Ret = 408
		return
	}

	fieldCode := this.GetString("FieldCode")
	if fieldCode == "" {
		br.Msg = "请选择指标"
		return
	}
	pageSize, _ := this.GetInt("PageSize")
	currentIndex, _ := this.GetInt("CurrentIndex")
	sort, _ := this.GetInt("Sort")

	var total int

	var startSize int
	if pageSize <= 0 {
		pageSize = utils.PageSize20
	}
	if currentIndex <= 0 {
		currentIndex = 1
	}

	startSize = utils.StartIndex(currentIndex, pageSize)

	total, err := data_input.GetInfoDataCount(fieldCode)
	if err != nil && err.Error() != utils.ErrNoRow() {
		br.Msg = "获取失败"
		br.Msg = "获取指标数据总数失败:Err:" + err.Error()
		return
	}
	list, err := data_input.GetInfoDataList(fieldCode, startSize, pageSize, sort)
	if err != nil && err.Error() != utils.ErrNoRow() {
		br.Msg = "获取失败"
		br.Msg = "获取指标数据失败:Err:" + err.Error()
		return
	}
	page := paging.GetPaging(currentIndex, pageSize, total)
	resp := new(data_input.FieldDataListResp)
	resp.List = list
	resp.Paging = page
	br.Ret = 200
	br.Success = true
	br.Msg = "获取成功"
	br.Data = resp
}

// @Title 删除指标数据
// @Description 删除指标数据接口
// @Param	request	body data_input.FieldDataDeleteReq true "type json string"
// @Success Ret=200 删除成功
// @router /field_data/delete [post]
func (this *DataInputController) FieldDataDelete() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()

	sysUser := this.SysUser
	if sysUser == nil {
		br.Msg = "请登录"
		br.ErrMsg = "请登录，SysUser Is Empty"
		br.Ret = 408
		return
	}

	var req data_input.FieldDataDeleteReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}
	err = data_input.DeleteFieldData(req.Id)
	if err != nil {
		br.Msg = "删除失败！"
		br.ErrMsg = "删除失败，Err:" + err.Error()
		return
	}
	{
		fieldItem, _ := data_input.GetFieldInfoByFieldCode(req.TradeCode)
		if fieldItem != nil {
			needModifyDate, _ := services.GetDateByFrequency(fieldItem.DateType)
			maxDate, minDate, _ := data_input.GetMaxAndMinDate(req.TradeCode)
			status, _ := services.CheckFieldInfoStatus(req.TradeCode)
			data_input.ModifyFieldInfoStatus(req.TradeCode, maxDate, minDate, needModifyDate, status)
		}
	}

	br.Ret = 200
	br.Success = true
	br.Msg = "删除成功"
}

// @Title 保存指标数据
// @Description 保存指标数据接口
// @Param	request	body data_input.FieldDataSaveReq true "type json string"
// @Success Ret=200 保存成功
// @router /field_data/save [post]
func (this *DataInputController) FieldDataSave() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()

	sysUser := this.SysUser
	if sysUser == nil {
		br.Msg = "请登录"
		br.ErrMsg = "请登录，SysUser Is Empty"
		br.Ret = 408
		return
	}

	var req data_input.FieldDataSaveReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}
	if req.FieldCode == "" {
		br.Msg = "请选择指标！"
		return
	}
	fieldItem, _ := data_input.GetFieldInfoByFieldCode(req.FieldCode)
	if fieldItem == nil {
		br.Msg = "请选择指标！"
		br.ErrMsg = "指标信息不存在"
		return
	}
	if req.Id > 0 {
		err = data_input.ModifyInfoData(req.Dt, req.Close, req.Id)
		if err != nil {
			br.Msg = "保存失败！"
			br.ErrMsg = "修改数据失败，Err:" + err.Error()
			return
		}
	} else {
		count, err := data_input.GetInfoDataCountByDate(req.FieldCode, req.Dt)
		if err != nil && err.Error() != utils.ErrNoRow() {
			br.Msg = "保存失败！"
			br.ErrMsg = "修改数据失败，Err:" + err.Error()
			return
		}
		if count > 0 {
			err = data_input.ModifyInfoData(req.Dt, req.Close, req.Id)
			if err != nil {
				br.Msg = "保存失败！"
				br.ErrMsg = "修改数据失败，Err:" + err.Error()
				return
			}
		} else {
			item := new(data_input.InfoData)
			item.ModifyTime = time.Now().Format(utils.FormatDateTime)
			item.Close = req.Close
			item.Dt = req.Dt
			item.TradeCode = req.FieldCode
			err = data_input.AddInfoData(item)
			if err != nil {
				br.Msg = "保存失败！"
				br.ErrMsg = "新增数据失败，Err:" + err.Error()
				return
			}
		}
	}

	//更新指标状态
	status, err := services.CheckFieldInfoStatus(req.FieldCode)
	if err != nil {
		br.Msg = "暂停失败！"
		br.Msg = "判断指标状态失败,Err:" + err.Error()
		return
	}
	maxDate, minDate, err := data_input.GetMaxAndMinDate(req.FieldCode)
	needModifyDate, _ := services.GetDateByFrequency(fieldItem.DateType)
	err = data_input.ModifyFieldInfoStatus(req.FieldCode, maxDate, minDate, needModifyDate, status)
	if err != nil {
		br.Msg = "暂停失败！"
		br.Msg = "修改指标状态失败,Err:" + err.Error()
		return
	}

	br.Ret = 200
	br.Success = true
	br.Msg = "保存成功"
	br.Data = status
}

// @Title 暂停/启用数据更新
// @Description 暂停/启用数据更新接口
// @Param	request	body data_input.SuspendEnableSaveReq true "type json string"
// @Success Ret=200 保存成功
// @router /suspend_enable/save [post]
func (this *DataInputController) SuspendEnableSave() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()

	sysUser := this.SysUser
	if sysUser == nil {
		br.Msg = "请登录"
		br.ErrMsg = "请登录，SysUser Is Empty"
		br.Ret = 408
		return
	}

	var req data_input.SuspendEnableSaveReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}
	if req.FieldCode == "" {
		br.Msg = "请选择指标！"
		return
	}
	fieldItem, _ := data_input.GetFieldInfoByFieldCode(req.FieldCode)
	if fieldItem == nil {
		br.Msg = "请选择指标！"
		br.ErrMsg = "指标信息不存在"
		return
	}

	status := req.Status
	if status <= 0 {
		br.Msg = "无效的状态码！"
		return
	}
	needModifyDate, _ := services.GetDateByFrequency(fieldItem.DateType)
	maxDate, minDate, err := data_input.GetMaxAndMinDate(req.FieldCode)
	if status != 3 {
		err = data_input.ModifyFieldInfoStatus(req.FieldCode, maxDate, minDate, needModifyDate, 3)
		if err != nil {
			br.Msg = "暂停失败！"
			br.Msg = "暂停失败,Err:" + err.Error()
			return
		}
	} else {
		status, err := services.CheckFieldInfoStatus(req.FieldCode)
		if err != nil {
			br.Msg = "暂停失败！"
			br.Msg = "判断指标状态失败,Err:" + err.Error()
			return
		}
		err = data_input.ModifyFieldInfoStatus(req.FieldCode, maxDate, minDate, needModifyDate, status)
		if err != nil {
			br.Msg = "暂停失败！"
			br.Msg = "修改指标状态失败,Err:" + err.Error()
			return
		}
	}
	br.Ret = 200
	br.Success = true
	br.Msg = "保存成功"
}
