package system

import (
	"encoding/base64"
	"encoding/json"
	"hongze/equity_admin/controllers"
	"hongze/equity_admin/models"
	"hongze/equity_admin/models/system"
	"hongze/equity_admin/utils"
	"rdluck_tools/paging"
	"strings"
	"time"
)

//系统用户
type SysAdminController struct {
	controllers.BaseAuthController
}

// @Title 根据分组或者部门获取系统用户列表
// @Description 根据分组或者部门获取系统用户列表接口
// @Param   PageSize   query   int  true       "每页数据条数"
// @Param   CurrentIndex   query   int  true       "当前页页码，从1开始"
// @Param   DepartmentId   query   int  true       "部门ID"
// @Param   GroupId   query   int  true       "分组ID"
// @Param   KeyWord   query   string  true       "搜索关键词"
// @Success 200 {object} system.SysuserListResp
// @router /sysuser/list [get]
func (this *SysAdminController) ListSysuser() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()
	sysUser := this.SysUser
	if sysUser == nil {
		br.Msg = "请登录"
		br.ErrMsg = "请登录，SysUser Is Empty"
		return
	}
	pageSize, _ := this.GetInt("PageSize")
	currentIndex, _ := this.GetInt("CurrentIndex")
	departmentId, _ := this.GetInt("DepartmentId")
	groupId, _ := this.GetInt("GroupId")
	keyWord := this.GetString("KeyWord")

	var startSize int
	if pageSize <= 0 {
		pageSize = utils.PageSize20
	}
	if currentIndex <= 0 {
		currentIndex = 1
	}
	startSize = utils.StartIndex(currentIndex, pageSize)

	var condition string
	var pars []interface{}

	if departmentId > 0 {
		condition += ` AND department_id=? `
		pars = append(pars, departmentId)
	}

	if groupId > 0 {
		condition += ` AND group_id=? `
		pars = append(pars, groupId)
	}

	if keyWord != "" {
		condition += ` AND (real_name LIKE '%` + keyWord + `%' OR admin_name LIKE '%` + keyWord + `%' OR mobile LIKE '%` + keyWord + `%' )  `
	}

	total, err := system.GetSysuserListCount(condition, pars)
	if err != nil {
		br.Msg = "获取失败"
		br.ErrMsg = "获取失败，Err:" + err.Error()
		return
	}

	list, err := system.GetSysuserList(condition, pars, startSize, pageSize)
	if err != nil {
		br.Msg = "获取失败"
		br.ErrMsg = "获取失败，Err:" + err.Error()
		return
	}
	lenList := len(list)
	for i := 0; i < lenList; i++ {
		item := list[i]
		if item.DepartmentName != "" {
			if item.GroupName == "" {
				list[i].DepartmentGroup = item.DepartmentName
			} else {
				list[i].DepartmentGroup = item.DepartmentName + "/" + item.GroupName
			}
		}
		labelVal := 0
		if item.RoleTypeCode == utils.ROLE_TYPE_CODE_ADMIN {
			labelVal = 1
		} else if item.RoleTypeCode == utils.ROLE_TYPE_CODE_FICC_ADMIN ||
			item.RoleTypeCode == utils.ROLE_TYPE_CODE_RAI_ADMIN {
			labelVal = 2
		} else {
			if item.Authority == 1 {
				labelVal = 3
			} else if item.Authority == 2 {
				labelVal = 4
			}
		}
		list[i].LabelVal = labelVal
	}
	page := paging.GetPaging(currentIndex, pageSize, total)
	resp := new(system.SysuserListResp)
	resp.List = list
	resp.Paging = page
	br.Ret = 200
	br.Success = true
	br.Msg = "获取成功"
	br.Data = resp
}

// @Title 新增系统用户
// @Description 新增系统用户接口
// @Param	request	body system.SysuserAddReq true "type json string"
// @Success 200 新增成功
// @router /sysuser/add [post]
func (this *SysAdminController) Add() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()
	var req system.SysuserAddReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}

	if req.RoleId <= 0 {
		br.Msg = "请选择角色"
		br.ErrMsg = "角色ID小于等于0"
		return
	}

	count, err := system.GetSysAdminCount(req.AdminName)
	if err != nil {
		br.Msg = "获取数据失败"
		br.ErrMsg = "获取数据失败"
		return
	}
	if count > 0 {
		br.Msg = "账号已存在,请重新输入"
		br.IsSendEmail = false
		return
	}
	if req.Mobile != "" {
		mobileCount, err := system.GetSysAdminCountByMobile(req.Mobile)
		if err != nil {
			br.Msg = "判断手机号是否存在失败"
			br.ErrMsg = "判断手机号是否存在失败，Err:" + err.Error()
			return
		}
		if mobileCount > 0 {
			br.Msg = "手机号已存在，请重新填写"
			br.IsSendEmail = false
			return
		}
	}
	//req.Mobile

	var roleName, departmentName, groupName string

	roleItem, err := system.GetSysRoleById(req.RoleId)
	if err != nil {
		if err.Error() == utils.ErrNoRow() {
			br.Msg = "所选角色不存在"
			return
		}
		br.Msg = "获取数据失败"
		br.ErrMsg = "获取角色数据失败，Err:" + err.Error()
		return
	}
	if roleItem != nil {
		roleName = roleItem.RoleName
	}
	if req.DepartmentId > 0 {
		departmentItem, err := system.GetSysDepartmentById(req.DepartmentId)
		if err != nil {
			if err.Error() == utils.ErrNoRow() {
				br.Msg = "所选部门不存在"
				return
			}
			br.Msg = "获取数据失败"
			br.ErrMsg = "获取部门数据失败，Err:" + err.Error()
			return
		}
		if departmentItem != nil {
			departmentName = departmentItem.DepartmentName
		}
	}

	if req.GroupId > 0 {
		groupItem, err := system.GetSysGroupById(req.GroupId)
		if err != nil {
			if err.Error() == utils.ErrNoRow() {
				br.Msg = "所选分组不存在"
				return
			}
			br.Msg = "获取数据失败"
			br.ErrMsg = "获取分组数据失败，Err:" + err.Error()
			return
		}
		if groupItem != nil {
			groupName = groupItem.GroupName
		}
	}

	pwdByte, err := base64.StdEncoding.DecodeString(req.Password)
	if err != nil {
		br.Msg = "解析数据失败"
		br.ErrMsg = "解析数据失败，Err:" + err.Error()
		return
	}
	pwdStr := string(pwdByte)
	pwdStr = strings.ToLower(pwdStr)
	if pwdStr == "" {
		br.Msg = "请输入密码"
		return
	}
	if !utils.CheckPwd(pwdStr) {
		br.Msg = "密码格式错误，请重新输入"
		return
	}
	admin := new(system.Admin)
	admin.AdminName = req.AdminName
	admin.RealName = req.RealName
	admin.Password = utils.MD5(pwdStr)
	admin.LastUpdatedPasswordTime = time.Now().Format(utils.FormatDateTime)
	admin.Enabled = 1
	admin.LastLoginTime = time.Now().Format(utils.FormatDateTime)
	admin.CreatedTime = time.Now()
	admin.LastUpdatedTime = time.Now().Format(utils.FormatDateTime)
	admin.Mobile = req.Mobile
	admin.RoleType = 0
	admin.RoleId = req.RoleId
	admin.RoleName = roleName
	admin.RoleTypeCode = roleItem.RoleTypeCode
	admin.DepartmentId = req.DepartmentId
	admin.DepartmentName = departmentName
	admin.GroupId = req.GroupId
	admin.GroupName = groupName
	if strings.Contains(admin.RoleTypeCode, "researcher") {
		admin.Role = "researcher"
	} else if strings.Contains(admin.RoleTypeCode, "seller") {
		admin.Role = "sales"
	} else {
		admin.Role = "admin"
	}

	var authority int
	if roleItem.RoleTypeCode == utils.ROLE_TYPE_CODE_ADMIN {
		authority = 3
	} else {
		if strings.Contains(roleItem.RoleType, "组长") {
			authority = 2
		} else if strings.Contains(roleItem.RoleType, "部门经理") {
			authority = 1
		} else {
			authority = 0
		}
	}
	admin.Authority = authority
	admin.Position = req.Position
	err = system.AddAdmin(admin)
	if err != nil {
		br.Msg = "新增失败"
		br.ErrMsg = "新增失败，Err:" + err.Error()
		return
	}
	br.Ret = 200
	br.Success = true
	br.Msg = "新增成功"
}

// @Title 编辑系统用户
// @Description 编辑系统用户接口
// @Param	request	body system.SysuserEditReq true "type json string"
// @Success 200 编辑成功
// @router /sysuser/edit [post]
func (this *SysAdminController) Edit() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()
	var req system.SysuserEditReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}

	if req.RoleId <= 0 {
		br.Msg = "请选择角色"
		br.ErrMsg = "角色ID小于等于0"
		return
	}

	item, err := system.GetSysAdminByName(req.AdminName)
	if err != nil && err.Error() != utils.ErrNoRow() {
		br.Msg = "获取数据失败"
		br.ErrMsg = "获取系统用户数据失败，Err:" + err.Error() + ";AdminName:" + req.AdminName
		return
	}
	if item != nil && item.AdminId != req.AdminId {
		br.Msg = "账号名称已存在,请重新输入"
		return
	}
	if item != nil {
		if req.Mobile != "" && req.Mobile != item.Mobile {
			mobileCount, err := system.GetSysAdminCountByMobile(req.Mobile)
			if err != nil {
				br.Msg = "判断手机号是否存在失败"
				br.ErrMsg = "判断手机号是否存在失败，Err:" + err.Error()
				return
			}
			if mobileCount > 0 {
				br.Msg = "手机号已存在，请重新填写"
				br.IsSendEmail = false
				return
			}
		}
	} else {
		mobileCount, err := system.GetSysAdminCountByMobile(req.Mobile)
		if err != nil {
			br.Msg = "判断手机号是否存在失败"
			br.ErrMsg = "判断手机号是否存在失败，Err:" + err.Error()
			return
		}
		if mobileCount > 0 {
			br.Msg = "手机号已存在，请重新填写"
			br.IsSendEmail = false
			return
		}
	}

	var roleName, departmentName, groupName string

	roleItem, err := system.GetSysRoleById(req.RoleId)
	if err != nil {
		if err.Error() == utils.ErrNoRow() {
			br.Msg = "角色不存在，请重新选择"
			br.ErrMsg = "角色不存在"
			return
		}
		br.Msg = "获取数据失败"
		br.ErrMsg = "获取角色数据失败，Err:" + err.Error()
		return
	}
	if roleItem != nil {
		roleName = roleItem.RoleName
	}
	if req.DepartmentId > 0 {
		departmentItem, err := system.GetSysDepartmentById(req.DepartmentId)
		if err != nil {
			br.Msg = "获取数据失败"
			br.ErrMsg = "获取部门数据失败，Err:" + err.Error()
			return
		}
		if departmentItem != nil {
			departmentName = departmentItem.DepartmentName
		}
	}

	if req.GroupId > 0 {
		groupItem, err := system.GetSysGroupById(req.GroupId)
		if err != nil {
			br.Msg = "获取数据失败"
			br.ErrMsg = "获取分组数据失败，Err:" + err.Error()
			return
		}
		if groupItem != nil {
			groupName = groupItem.GroupName
		}
	}

	pwdByte, err := base64.StdEncoding.DecodeString(req.Password)
	if err != nil {
		br.Msg = "解析数据失败"
		br.ErrMsg = "解析数据失败，Err:" + err.Error()
		return
	}
	adminItem, err := system.GetSysAdminById(req.AdminId)
	if err != nil {
		br.Msg = "获取数据失败"
		br.ErrMsg = "获取系统用户数据失败，Err:" + err.Error()
		return
	}
	var newPwd string
	pwdStr := string(pwdByte)
	if item.Password != pwdStr {
		pwdStr = strings.ToLower(pwdStr)
		if pwdStr == "" {
			if adminItem == nil {
				br.Msg = "系统用户信息不存在"
				return
			}
			newPwd = adminItem.Password
		} else {
			if !utils.CheckPwd(pwdStr) {
				br.Msg = "密码格式错误，请重新输入"
				return
			}
			newPwd = utils.MD5(pwdStr)
		}
	} else {
		newPwd = adminItem.Password
	}
	admin := new(system.Admin)
	admin.AdminId = req.AdminId
	admin.AdminName = req.AdminName
	admin.RealName = req.RealName
	admin.Password = newPwd
	admin.LastUpdatedPasswordTime = time.Now().Format(utils.FormatDateTime)
	admin.Enabled = 1
	admin.LastLoginTime = time.Now().Format(utils.FormatDateTime)
	admin.CreatedTime = time.Now()
	admin.LastUpdatedTime = time.Now().Format(utils.FormatDateTime)
	admin.Mobile = req.Mobile
	admin.RoleType = 0
	admin.RoleId = req.RoleId
	admin.RoleName = roleName
	admin.DepartmentId = req.DepartmentId
	admin.DepartmentName = departmentName
	admin.GroupId = req.GroupId
	admin.GroupName = groupName
	var authority int
	if roleItem.RoleTypeCode == utils.ROLE_TYPE_CODE_ADMIN {
		authority = 3
	} else {
		if strings.Contains(roleItem.RoleType, "组长") {
			authority = 2
		} else if strings.Contains(roleItem.RoleType, "部门经理") {
			authority = 1
		} else {
			authority = 0
		}
	}
	admin.Authority = authority
	admin.Position = req.Position
	admin.RoleTypeCode = roleItem.RoleTypeCode
	err = system.EditAdmin(admin)
	if err != nil {
		br.Msg = "编辑失败"
		br.ErrMsg = "编辑失败，Err:" + err.Error()
		return
	}
	br.Ret = 200
	br.Success = true
	br.Msg = "编辑成功"
}

// @Title 删除系统用户
// @Description 删除系统用户接口
// @Param	request	body system.SysuserDeleteReq true "type json string"
// @Success 200 删除成功
// @router /sysuser/delete [post]
func (this *SysAdminController) Delete() {
	br := new(models.BaseResponse).Init()
	defer func() {
		this.Data["json"] = br
		this.ServeJSON()
	}()
	var req system.SysuserDeleteReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}
	if req.AdminId <= 0 {
		br.Msg = "参数错误"
		br.ErrMsg = "参数错误,GroupId 小于等于0 "
		return
	}
	err = system.DeleteSysuser(req.AdminId)
	if err != nil {
		br.Msg = "删除失败"
		br.ErrMsg = "删除失败，Err:" + err.Error()
		return
	}
	br.Ret = 200
	br.Success = true
	br.Msg = "删除成功"
}
//
//func init() {
//	fmt.Println("start")
//	admin := new(system.Admin)
//	admin.AdminName = "admin"
//	admin.RealName = "超级管理员"
//	admin.Password = utils.MD5("123456")
//	admin.LastUpdatedPasswordTime = time.Now().Format(utils.FormatDateTime)
//	admin.Enabled = 1
//	admin.LastLoginTime = time.Now().Format(utils.FormatDateTime)
//	admin.CreatedTime = time.Now()
//	admin.LastUpdatedTime = time.Now().Format(utils.FormatDateTime)
//	admin.Mobile = ""
//	admin.RoleType = 0
//	admin.RoleId = 1
//	admin.RoleName = "超级管理员"
//	admin.RoleTypeCode = "admin"
//	admin.DepartmentId = 1
//	admin.DepartmentName = "超级管理员"
//	admin.GroupId = 1
//	admin.GroupName = "超级管理员"
//	if strings.Contains(admin.RoleTypeCode, "researcher") {
//		admin.Role = "researcher"
//	} else if strings.Contains(admin.RoleTypeCode, "seller") {
//		admin.Role = "sales"
//	} else {
//		admin.Role = "admin"
//	}
//
//	var authority int
//	authority = 0
//	admin.Authority = authority
//	admin.Position = "超级管理员"
//	err := system.AddAdmin(admin)
//	fmt.Println("err:", err)
//	fmt.Println("end")
//}
